<?php

namespace Soumettre;

use Soumettre\Exceptions\PluginInactifException;

require_once( realpath( __DIR__ . '/../sdk-api-php/src/SoumettreServices.php' ) );
require_once( realpath( __DIR__ . '/../sdk-api-php/src/SoumettreApiClient.php' ) );
require_once( realpath( __DIR__ . '/../sdk-api-php/src/SoumettreApi.php' ) );
require_once( realpath( __DIR__ . '/../sdk-api-php/src/Exceptions.php' ) );

class SoumettreWP extends SoumettreApiClient {
    protected $mode = 'prod';
    protected $available_services = array(
        'check_api',
        'check_added',
        'categories',
        'post',
        'delete',
        'fetch',
        'update'
    );

    function __construct() {
        parent::__construct();

        $this->endpoint = 'https://soumettre.fr/api/';
//        $this->endpoint = 'http://soumettre.dev/api/';

        $this->wp_set_credentials();

        try {
            $this->is_plugin_active();
            $this->check_request();
        } catch ( \Exception $e ) {
            echo json_encode( array( 'error' => get_class( $e ), 'message' => $e->getMessage() ) );
        }
    }

    public function is_plugin_active() {
        $active_plugins = apply_filters( 'active_plugins', get_option( 'active_plugins' ) );

        foreach ( $active_plugins as $active_plugin ) {
            if ( strpos( $active_plugin, 'soumettre-partner' ) !== false ) {
                return true;
            }
        }

        throw new PluginInactifException();
    }

    /**
     * Charge les credentials depuis la base de données (utilisé par Wordpress)
     *
     * @param string $prefix Préfixe SQL pour les options
     */
    public function wp_set_credentials( $prefix = 'soum_sour_' ) {
        $this->email      = get_option( $prefix . 'email' );
        $this->api_key    = get_option( $prefix . 'api_key' );
        $this->api_secret = get_option( $prefix . 'api_secret' );
        $this->author     = get_option( $prefix . 'author' );
    }

    protected function check_api() {
        $default_headers = array(
            'Name'        => 'Plugin Name',
            'PluginURI'   => 'Plugin URI',
            'Version'     => 'Version',
            'Description' => 'Description',
            'Author'      => 'Author',
            'AuthorURI'   => 'Author URI',
            'TextDomain'  => 'Text Domain',
            'DomainPath'  => 'Domain Path',
            'Network'     => 'Network',
            '_sitewide'   => 'Site Wide Only',
        );

        $plugin_data = get_file_data( dirname( __FILE__ ) . '/../soumettre-partner.php', $default_headers, 'plugin' );

        header( 'Content-Type: application/json' );
        echo json_encode( array( 'status' => 'ok', 'version' => $plugin_data['Version'] ) );
    }

    protected function check_request() {
        if ( ! isset( $_GET['method'] ) ) {
            return true;
        }

        $service = $_GET['method'];
        $params  = ( $this->mode == 'test' ) ? $_GET : $_POST;

        if ( ! in_array( $service, $this->available_services ) ) {
            throw new \Exception( "Service inconnu" );
        }

        $this->service = $service;
        $this->params  = $params;

        $this->check_signature( $service, $params );

        $this->$service( $params );
    }

    public function request( $service, $post_params = array() ) {
        $endpoint    = $this->endpoint . $service;
        $post_params = $this->sign( $service, $post_params );

        $res = wp_remote_post( $endpoint, array( 'body' => $post_params ) );
        if ( is_object( $res ) && get_class( $res ) == 'WP_Error' ) {
            return array( 'status' => 'error', 'message' => $res->get_error_message() );
        }

        return json_decode( $res['body'] );
    }

    public function site_add( $url = null, $cms = null, $endpoint = null ) {
        $params = array(
            'url' => $url,
            'cms' => $cms
        );

        if ( $endpoint ) {
            $params['endpoint'] = $endpoint;
        }

        $res = $this->request( 'site_register', $params );

        return $res;
    }

    public function check_added( $params ) {
        $args = array(
            'post__not_in'        => get_option( "sticky_posts" ),
            'ignore_sticky_posts' => 1,
            'meta_query'          => array(
                array(
                    'key'     => get_option( 'soum_sour_url_field', true ),
                    'value'   => $params['url'],
                    'compare' => '=',
                )
            )
        );

        $meta_query = new WP_Query( $args );
        if ( $meta_query->have_posts() ) {
            while ( $meta_query->have_posts() ) {
                $meta_query->the_post();
                echo json_encode( array( 'status' => 'found', 'url' => get_permalink() ) );
                die();
            }
        } else {
            echo json_encode( array( 'status' => 'not_found' ) );
            die();
        }
    }

    public function categories( $params ) {
        $ret        = array();
        $categories = get_categories( array( 'taxonomy' => 'category' ) );
        foreach ( $categories as $category ) {
            $parent = $category->parent != 0 ? $category->parent : '#';
            $ret[]  = array( 'id' => $category->term_id, 'text' => $category->name, 'parent' => $parent );
        }
        echo json_encode( $ret );
    }

    public function test() {
        $res = $this->request( 'test' );

        echo json_encode( $res );
        die();
    }

    public function uploadAndReplaceImages( $content, $post_id )
	{
        $imagesToUpload = [];
        $imagesUploaded = [];

        $content = stripslashes($content);

        preg_match_all('#<img[^>]+src="([^">]+)"#', $content, $images);

        if( !count($images) ) {
            return false;
        }

        foreach( $images[1] as $image ) {
            if( strpos($image, '/upload/t-') !== false ) {
                $imagesToUpload[] = $image;
            }
        }
        if ( ! function_exists( 'wp_crop_image' ) ) {
            include( ABSPATH . 'wp-admin/includes/image.php' );
        }


        foreach( $imagesToUpload as $img ) {
            $info = pathinfo($img);
            $upload = wp_upload_bits( $info['basename'], null, file_get_contents($img) );

            if ( $upload['error'] ) {
                continue;
            }

            $file = $upload['file'];

            $wp_filetype = wp_check_filetype(basename($file), null );
            $attachment = array(
                'post_mime_type' => $wp_filetype['type'],
                'post_title' => preg_replace('/\.[^.]+$/', '', basename($file)),
                'post_content' => '',
                'post_status' => 'inherit'
            );
            $attach_id = wp_insert_attachment( $attachment, $file, $post_id );
            $attach_data = wp_generate_attachment_metadata( $attach_id, $file );
            wp_update_attachment_metadata( $attach_id,  $attach_data );
            $imagesUploaded[$img] = $upload['url'];
        }


        if( count($imagesUploaded) ) {
            $content = str_replace( array_keys($imagesUploaded), array_values($imagesUploaded), $content );
            wp_update_post( ['ID'=>$post_id, 'post_content'=>$content] );
        }
		

		return $imagesUploaded;
	}

    public function post( $params ) {
        $meta_input = array();

        $meta_input[ get_option( 'soum_sour_url_field' ) ] = $params['url'];
        foreach ( $params as $p_key => $p_val ) {
            if ( strpos( $p_key, 'custom_' ) === 0 ) {
                $key                = str_replace( 'custom_', '', $p_key );
                $meta_input[ $key ] = strip_tags( $p_val );
            }
        }

        if ( isset( $params['draft'] ) ) {
            $post_status = 'draft';
        } else {
            $post_status = 'publish';
        }

        $post_arr = array(
            'post_status'   => $post_status,
            'post_title'    => $params['title'],
            'post_content'  => $params['content'],
            'post_category' => array( $params['category'] ),
            'post_author'   => get_option( 'soum_sour_author' ),
            'meta_input'    => $meta_input
        );

        $post_ID = wp_insert_post( $post_arr );

        if ( is_numeric( $post_ID ) && $post_ID > 0 ) {

            add_post_meta( $post_ID, '_soumettre_task_id', $params['task_id'], true );

            $this->uploadAndReplaceImages( $params['content'], $post_ID );

            if ( isset( $params['image'] ) ) {
                $this->set_featured_image( $params['image'], $post_ID );
                $this->set_directorypress_image_fields( $params['image'], $post_ID );
            }

            echo json_encode( array( 'status' => 'ok', 'id' => $post_ID, 'url' => get_permalink( $post_ID ) ) );
        } else {

            echo json_encode( array( 'status' => 'error', 'post_arr' => $post_arr ) );
        }
    }

    protected function set_directorypress_image_fields( $image_url, $post_id ) {
        $upload_dir = wp_upload_dir();
        $image_data = file_get_contents( $image_url );
        $filename   = basename( $image_url );
        if ( wp_mkdir_p( $upload_dir['path'] ) ) {
            $file = $upload_dir['path'] . '/' . $filename;
        } else {
            $file = $upload_dir['basedir'] . '/' . $filename;
        }
        file_put_contents( $file, $image_data );

        if ( wp_mkdir_p( $upload_dir['path'] ) ) {
            $image_url = get_home_url() . $upload_dir['path'] . '/' . $filename;
        } else {
            $image_url = get_home_url() . $upload_dir['basedir'] . '/' . $filename;
        }

        add_post_meta( $post_id, 'image', $image_url, true );
    }

    protected function set_featured_image( $image_url, $post_id ) {
        $upload_dir = wp_upload_dir();
        $image_data = file_get_contents( $image_url );
        $filename   = basename( $image_url );
        if ( wp_mkdir_p( $upload_dir['path'] ) ) {
            $file = $upload_dir['path'] . '/' . $filename;
        } else {
            $file = $upload_dir['basedir'] . '/' . $filename;
        }
        file_put_contents( $file, $image_data );

        $wp_filetype = wp_check_filetype( $filename, null );
        $attachment  = array(
            'post_mime_type' => $wp_filetype['type'],
            'post_title'     => sanitize_file_name( $filename ),
            'post_content'   => '',
            'post_status'    => 'inherit'
        );
        $attach_id   = wp_insert_attachment( $attachment, $file, $post_id );
        require_once( ABSPATH . 'wp-admin/includes/image.php' );
        $attach_data = wp_generate_attachment_metadata( $attach_id, $file );
        $res1        = wp_update_attachment_metadata( $attach_id, $attach_data );
        $res2        = set_post_thumbnail( $post_id, $attach_id );
    }


    public function fetch( $params ) {
        header( 'Content-Type: application/json' );

        $task_ID = $params['task_id'];
        $url     = $params['url'];
        $post    = false;

        if ( ! is_numeric( $task_ID ) && strlen( $url ) < 5 ) {
            echo json_encode( array( 'status' => 'error', 'message' => 'wrong parameters' ) );

            return false;
        }

        // get post by task_id
        if ( is_numeric( $task_ID ) ) {
            $post = $this->get_post_by_task_id( $task_ID );
        }
        if ( ! $post ) {
            $post = $this->get_post_by_url( $url );
        }

        if ( ! $post ) {
            echo json_encode( array( 'status' => 'error', 'message' => 'no posts' ) );

            return false;
        }

        echo json_encode( $post );
        die();
    }

    private function get_post_by_task_id( $task_ID ) {
        $loop = new \WP_Query( array(
            'ignore_sticky_posts' => 1,
            'meta_key' => '_soumettre_task_id',
            'meta_value' => $task_ID
        ) );
        if ( ! $loop->have_posts() ) {
            return false;
        }

        $loop->the_post();

        global $post;

        return $post;
    }

    private function get_post_by_url( $url ) {
        $post_id = url_to_postid( $url );
        $post    = get_post( $post_id );

        return $post;
    }

    public function update( $params ) {
        header( 'Content-Type: application/json' );

        $task_ID = $params['task_id'];
        $url     = $params['url'];
        $post    = false;

        if ( ! is_numeric( $task_ID ) && strlen( $url ) < 5 ) {
            echo json_encode( array( 'status' => 'error', 'message' => 'wrong parameters' ) );

            return false;
        }

        // get post by task_id
        if ( is_numeric( $task_ID ) ) {
            $post = $this->get_post_by_task_id( $task_ID );
        }
        if ( ! $post ) {
            $post = $this->get_post_by_url( $url );
        }

        $post->post_title   = stripslashes($params['title']);
        $post->post_content = stripslashes($params['editedContent']);

        if ( wp_update_post( $post ) ) {
            echo json_encode( array( 'status' => 'ok' ) );
            die();
        }

        echo json_encode( array( 'status' => 'error', 'message' => 'no_update' ) );
        die();
    }
}
