<?php
/**
 * @link              https://soumettre.fr/
 * @package           SoumettrePartner
 *
 * @wordpress-plugin
 * Plugin Name:       Soumettre Partner
 * Plugin URI:        https://soumettre.fr/webmasters
 * Description:       Adds your site on Soumettre.fr
 * Version:           1.7.0
 * Author:            Didier Sampaolo
 * Author URI:        https://didcode.com/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       soumettre-partner
 */


/*
 * @see : http://jeremypry.com/wordpress-plugin-development-with-github/#set-up-the-plugin-on-github
 */

// If this file is called directly, abort.
if (!defined('WPINC') || !defined('ABSPATH')) {
    die;
}

define('SOUMETTRE_API_URL', plugin_dir_url(__FILE__) . 'api/');

if (is_admin()) {
    add_action('init', 'call_SoumettrePartner_Admin');
}

function call_SoumettrePartner_Admin()
{
    new SoumettrePartner_Admin();
}

class SoumettrePartner_Admin
{
    protected $endpoint = '';
    protected $prefix = 'soum_sour_';

    protected $plugin_url;

    public function __construct()
    {

        add_action('admin_menu', array($this, 'admin_menu'));

        // ajax
        add_action('admin_footer', array($this, 'ajax'));
        add_action('wp_ajax_soumettre_save_options', array($this, 'ajax_save_options'));

    }

    public function ajax_save_options()
    {
        // sanitize and save options to database

        $api_key = $_POST['api_key'];
        if (strlen($api_key) > 42) {
            $api_key = substr($api_key, 0, 42);
        }
        update_option($this->prefix.'api_key', $api_key);

        $api_secret = $_POST['api_secret'];
        if (strlen($api_secret) > 42) {
            $api_secret = substr($api_secret, 0, 42);
        }

        $author_id = $_POST['author'];
        if (!is_numeric($author_id)) {
            $author_id = 1;
        }

        update_option($this->prefix.'api_secret', $api_secret);
        update_option($this->prefix.'email', sanitize_email($_POST['email']) );
        update_option($this->prefix.'author', $author_id);

        // register website on soumettre.fr
        require_once('inc/SoumettreWP.php');

        $api = new Soumettre\SoumettreWP();
        $res = $api->site_add(get_home_url(),
            'WordPress',
            plugin_dir_url(__FILE__) . 'api/index.php?method='
        );

        echo json_encode($res);
        die();
    }

    public function admin_menu()
    {
        add_menu_page(
            __('Soumettre Partner', 'soumettre-menu'),
            __('Soumettre Partner', 'soumettre-menu'),
            'manage_options',
            'soumettre-partner',
            array($this, 'admin_page'),
            plugin_dir_url(__FILE__) . 'favicon-16x16.png'
        );
    }

    public function admin_page()
    {
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }

        wp_enqueue_script('jquery-form');

        include('inc/admin_options.php');
    }

    public function ajax()
    { ?>
        <script type="text/javascript">
            jQuery(document).ready(function ($) {

                jQuery('#soumettre_save_changes').click(function (event) {
                    event.preventDefault();

                    var data = {
                        action: 'soumettre_save_options',
                        email: jQuery('#soumettre_email').val(),
                        api_key: jQuery('#soumettre_api_key').val(),
                        api_secret: jQuery('#soumettre_api_secret').val(),
                        author: jQuery('select[name="author"] option:selected').val()
                    };

                    jQuery.post(ajaxurl, data, function (response) {
                        if (response == null) {
                            $('#test_api_res').html('<span class="dashicons dashicons-no"></span> Une Erreur inconnue s\'est produite (response). :(');
                            return;
                        }

                        json = jQuery.parseJSON(response);

                        if (json == null) {
                            $('#test_api_res').html('<span class="dashicons dashicons-no"></span> Une Erreur inconnue s\'est produite. :(');
                            return;
                        }

                        if (json.status == 'OK') {
                            $('#test_api_res').html('<span class="dashicons dashicons-yes"></span>' + json.message);
                        } else {
                            $('#test_api_res').html('<span class="dashicons dashicons-no"></span>' + json.message);
                        }
                    });
                    return false;
                });
            });
        </script> <?php
    }
}

require __DIR__.'/plugin-update-checker/plugin-update-checker.php';
$myUpdateChecker = Puc_v4_Factory::buildUpdateChecker(
	'https://soumettre.fr/plugins/wordpress/metadata-partner.json',
	__FILE__, //Full path to the main plugin file or functions.php.
	'soumettre-partner'
);